# coding: utf-8

class ClientOptions(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'grant_user_permissions_access': 'bool'
    }

    attribute_serialization_map = {
        'grant_user_permissions_access': 'grantUserPermissionsAccess'
    }

    def __init__(self, grant_user_permissions_access=None):
        """ClientOptions"""
        self._grant_user_permissions_access = None
        if grant_user_permissions_access is not None:
            self.grant_user_permissions_access = grant_user_permissions_access

    @property
    def grant_user_permissions_access(self):
        """Gets the grant_user_permissions_access of this ClientOptions.

        Grant the client access to verify authorization on behalf of any user.

        :return: The grant_user_permissions_access of this ClientOptions.
        :rtype: bool
        """
        return self._grant_user_permissions_access

    @grant_user_permissions_access.setter
    def grant_user_permissions_access(self, grant_user_permissions_access):
        """Sets the grant_user_permissions_access of this ClientOptions.

        Grant the client access to verify authorization on behalf of any user.

        :param grant_user_permissions_access: The grant_user_permissions_access of this ClientOptions.
        :type: bool
        """

        self._grant_user_permissions_access = grant_user_permissions_access



# coding: utf-8

class ClientCollection(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'clients': 'list[Client]'
    }

    attribute_serialization_map = {
        'clients': 'clients'
    }

    def __init__(self, clients=None):
        """ClientCollection"""
        self._clients = None
        self.clients = clients

    @property
    def clients(self):
        """Gets the clients of this ClientCollection.

        A list of clients

        :return: The clients of this ClientCollection.
        :rtype: list[Client]
        """
        return self._clients

    @clients.setter
    def clients(self, clients):
        """Sets the clients of this ClientCollection.

        A list of clients

        :param clients: The clients of this ClientCollection.
        :type: list[Client]
        """
        if clients is None:
            raise ValueError("Invalid value for `clients`, must not be `None`")

        self._clients = clients



# coding: utf-8

class AccessRecordGroup(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'group_id': 'str'
    }

    attribute_serialization_map = {
        'group_id': 'groupId'
    }

    def __init__(self, group_id=None):
        """AccessRecordGroup"""
        self._group_id = None
        self.group_id = group_id

    @property
    def group_id(self):
        """Gets the group_id of this AccessRecordGroup.


        :return: The group_id of this AccessRecordGroup.
        :rtype: str
        """
        return self._group_id

    @group_id.setter
    def group_id(self, group_id):
        """Sets the group_id of this AccessRecordGroup.


        :param group_id: The group_id of this AccessRecordGroup.
        :type: str
        """
        if group_id is None:
            raise ValueError("Invalid value for `group_id`, must not be `None`")

        self._group_id = group_id



# coding: utf-8

class IdentityCollection(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'identities': 'list[Identity]'
    }

    attribute_serialization_map = {
        'identities': 'identities'
    }

    def __init__(self, identities=None):
        """IdentityCollection"""
        self._identities = None
        self.identities = identities

    @property
    def identities(self):
        """Gets the identities of this IdentityCollection.


        :return: The identities of this IdentityCollection.
        :rtype: list[Identity]
        """
        return self._identities

    @identities.setter
    def identities(self, identities):
        """Sets the identities of this IdentityCollection.


        :param identities: The identities of this IdentityCollection.
        :type: list[Identity]
        """
        if identities is None:
            raise ValueError("Invalid value for `identities`, must not be `None`")

        self._identities = identities



# coding: utf-8

class AccessRecordStatement(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'roles': 'list[str]',
        'resources': 'list[AccessRecordResource]',
        'users': 'list[AccessRecordUser]',
        'groups': 'list[AccessRecordGroup]'
    }

    attribute_serialization_map = {
        'roles': 'roles',
        'resources': 'resources',
        'users': 'users',
        'groups': 'groups',
    }

    def __init__(self, roles=None, resources=None, users=None, groups=None):
        """AccessRecordStatement"""
        self._roles = None
        self._resources = None
        self._users = None
        self._groups = None
        self.roles = roles
        self.resources = resources
        self.users = users
        self.groups = groups

    @property
    def roles(self):
        """Gets the roles of this AccessRecordStatement.


        :return: The roles of this AccessRecordStatement.
        :rtype: list[str]
        """
        return self._roles

    @roles.setter
    def roles(self, roles):
        """Sets the roles of this AccessRecordStatement.


        :param roles: The roles of this AccessRecordStatement.
        :type: list[str]
        """
        if roles is None:
            raise ValueError("Invalid value for `roles`, must not be `None`")

        self._roles = roles

    @property
    def resources(self):
        """Gets the resources of this AccessRecordStatement.


        :return: The resources of this AccessRecordStatement.
        :rtype: list[AccessRecordResource]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this AccessRecordStatement.


        :param resources: The resources of this AccessRecordStatement.
        :type: list[AccessRecordResource]
        """
        if resources is None:
            raise ValueError("Invalid value for `resources`, must not be `None`")

        self._resources = resources

    @property
    def users(self):
        """Gets the users of this AccessRecordStatement.

        The list of users this record applies to

        :return: The users of this AccessRecordStatement.
        :rtype: list[AccessRecordUser]
        """
        return self._users

    @users.setter
    def users(self, users):
        """Sets the users of this AccessRecordStatement.

        The list of users this record applies to

        :param users: The users of this AccessRecordStatement.
        :type: list[AccessRecordUser]
        """
        self._users = users

    @property
    def groups(self):
        """Gets the groups of this AccessRecordStatement.

        The list of groups this record applies to. Users in these groups will be receive access to the resources listed.

        :return: The groups of this AccessRecordStatement.
        :rtype: list[AccessRecordGroup]
        """
        return self._groups

    @groups.setter
    def groups(self, groups):
        """Sets the groups of this AccessRecordStatement.

        The list of groups this record applies to. Users in these groups will be receive access to the resources listed.

        :param groups: The groups of this AccessRecordStatement.
        :type: list[AccessRecordGroup]
        """
        self._groups = groups


# coding: utf-8

class ResourcePermissionCollection(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'resources': 'list[ResourcePermission]'
    }

    attribute_serialization_map = {
        'resources': 'resources'
    }

    def __init__(self, resources=None):
        """ResourcePermissionCollection"""
        self._resources = None
        self.resources = resources

    @property
    def resources(self):
        """Gets the resources of this ResourcePermissionCollection.


        :return: The resources of this ResourcePermissionCollection.
        :rtype: list[ResourcePermission]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this ResourcePermissionCollection.


        :param resources: The resources of this ResourcePermissionCollection.
        :type: ResourcePermission
        """
        if resources is None:
            raise ValueError("Invalid value for `resources`, must not be `None`")

        self._resources = resources



# coding: utf-8
from __future__ import absolute_import

import json

# python 2 and python 3 compatibility library
from authress_sdk.http_client import HttpClient
from authress_sdk.api.access_records_api import AccessRecordsApi
from authress_sdk.api.accounts_api import AccountsApi
from authress_sdk.api.resource_permissions_api import ResourcePermissionsApi
from authress_sdk.api.service_clients_api import ServiceClientsApi
from authress_sdk.api.user_permissions_api import UserPermissionsApi
from authress_sdk.api import token_verifier

class AuthressClient(object):
    def __init__(self, host=None, access_key=None):
        self._host = host if host.startswith('http') else f"https://{host}"
        self._token_verifier = token_verifier.TokenVerifier()
        self._http_client = HttpClient(host=self._host, access_key=access_key)

    def set_token(self, token):
        self._http_client.set_token(token)

    def get_client_token(self) -> str:
        self._http_client._get_client_token()

    def verify_token(self, token):
      """Verify a user access token

        On successful verification the response is the decoded user identity JWT.
        On failure this raises an exception

        Note: This method makes HTTP calls to fetch public keys for token signature verification

        :param string token: (required) The user's access token to verify
        :raises: :class:`Exception`: Unauthorized

        :return: UserIdentityJWT
        """
      return self._token_verifier.verify_token(self._host, token)

    @property
    def users(self) -> UserPermissionsApi:
        """Gets the User permissions API to make HTTP Requests with."""
        return UserPermissionsApi(api_client=self._http_client)

    @property
    def records(self) -> AccessRecordsApi:
        """Gets the Access Records API to make HTTP Requests with."""
        return AccessRecordsApi(api_client=self._http_client)

    @property
    def accounts(self) -> AccountsApi:
        """Gets the Authress Accounts API to make HTTP Requests with."""
        return AccountsApi(api_client=self._http_client)

    @property
    def resources(self) -> ResourcePermissionsApi:
        """Gets the Resources API to make HTTP Requests with."""
        return ResourcePermissionsApi(api_client=self._http_client)

    @property
    def clients(self) -> ServiceClientsApi:
        """Gets the Service Clients API to make HTTP Requests with."""
        return ServiceClientsApi(api_client=self._http_client)


# coding: utf-8

class AccountCollection(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'accounts': 'list[Account]'
    }

    attribute_serialization_map = {
        'accounts': 'accounts'
    }

    def __init__(self, accounts=None):
        """AccountCollection"""
        self._accounts = None
        self.accounts = accounts

    @property
    def accounts(self):
        """Gets the accounts of this AccountCollection.


        :return: The accounts of this AccountCollection.
        :rtype: list[Account]
        """
        return self._accounts

    @accounts.setter
    def accounts(self, accounts):
        """Sets the accounts of this AccountCollection.


        :param accounts: The accounts of this AccountCollection.
        :type: list[Account]
        """
        self._accounts = accounts



# coding: utf-8

class Identity(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'issuer': 'str',
        'audience': 'str'
    }

    attribute_serialization_map = {
        'issuer': 'issuer',
        'audience': 'audience'
    }

    def __init__(self, issuer=None, audience=None):
        """Identity"""
        self._issuer = None
        self._audience = None
        self.issuer = issuer
        self.audience = audience

    @property
    def issuer(self):
        """Gets the issuer of this Identity.

        The issuer of the JWT token. This can be any OIDC compliant provider.

        :return: The issuer of this Identity.
        :rtype: str
        """
        return self._issuer

    @issuer.setter
    def issuer(self, issuer):
        """Sets the issuer of this Identity.

        The issuer of the JWT token. This can be any OIDC compliant provider.

        :param issuer: The issuer of this Identity.
        :type: str
        """
        if issuer is None:
            raise ValueError("Invalid value for `issuer`, must not be `None`")

        self._issuer = issuer

    @property
    def audience(self):
        """Gets the audience of this Identity.

        The target of the JWT tokens. This must be a sub target of the account app or the whole app

        :return: The audience of this Identity.
        :rtype: str
        """
        return self._audience

    @audience.setter
    def audience(self, audience):
        """Sets the audience of this Identity.

        The target of the JWT tokens. This must be a sub target of the account app or the whole app

        :param audience: The audience of this Identity.
        :type: str
        """
        if audience is None:
            raise ValueError("Invalid value for `audience`, must not be `None`")

        self._audience = audience



# coding: utf-8

class AccessRecordUser(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'user_id': 'str'
    }

    attribute_serialization_map = {
        'user_id': 'userId'
    }

    def __init__(self, user_id=None):
        """AccessRecordUser"""
        self._user_id = None
        self.user_id = user_id

    @property
    def user_id(self):
        """Gets the user_id of this AccessRecordUser.


        :return: The user_id of this AccessRecordUser.
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this AccessRecordUser.


        :param user_id: The user_id of this AccessRecordUser.
        :type: str
        """
        if user_id is None:
            raise ValueError("Invalid value for `user_id`, must not be `None`")

        self._user_id = user_id



# coding: utf-8

class IdentityRequest(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'jwt': 'str',
        'preferred_audience': 'str'
    }

    attribute_serialization_map = {
        'jwt': 'jwt',
        'preferred_audience': 'preferredAudience'
    }

    def __init__(self, jwt=None, preferred_audience=None):
        """IdentityRequest"""
        self._jwt = None
        self._preferred_audience = None
        self.jwt = jwt
        if preferred_audience is not None:
            self.preferred_audience = preferred_audience

    @property
    def jwt(self):
        """Gets the jwt of this IdentityRequest.

        A valid JWT OIDC compliant token which will still pass authentication requests to the identity provider. Must contain a unique audience and issuer.

        :return: The jwt of this IdentityRequest.
        :rtype: str
        """
        return self._jwt

    @jwt.setter
    def jwt(self, jwt):
        """Sets the jwt of this IdentityRequest.

        A valid JWT OIDC compliant token which will still pass authentication requests to the identity provider. Must contain a unique audience and issuer.

        :param jwt: The jwt of this IdentityRequest.
        :type: str
        """
        if jwt is None:
            raise ValueError("Invalid value for `jwt`, must not be `None`")

        self._jwt = jwt

    @property
    def preferred_audience(self):
        """Gets the preferred_audience of this IdentityRequest.

        If the `jwt` token contains more than one valid audience, then the single audience that should associated with Authress. If more than one audience is preferred, repeat this call with each one.

        :return: The preferred_audience of this IdentityRequest.
        :rtype: str
        """
        return self._preferred_audience

    @preferred_audience.setter
    def preferred_audience(self, preferred_audience):
        """Sets the preferred_audience of this IdentityRequest.

        If the `jwt` token contains more than one valid audience, then the single audience that should associated with Authress. If more than one audience is preferred, repeat this call with each one.

        :param preferred_audience: The preferred_audience of this IdentityRequest.
        :type: str
        """

        self._preferred_audience = preferred_audience



# coding: utf-8

class AccessRecordResource(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'resource_uri': 'str'
    }

    attribute_serialization_map = {
        'resource_uri': 'resourceUri'
    }

    def __init__(self, resource_uri=None):
        """AccessRecordResource"""
        self._resource_uri = None
        self.resource_uri = resource_uri

    @property
    def resource_uri(self):
        """Gets the resource_uri of this AccessRecordResource.

        A resource path which can be top level, fully qualified, or end with a *. Parent resources imply permissions to sub resources.

        :return: The resource_uri of this AccessRecordResource.
        :rtype: str
        """
        return self._resource_uri

    @resource_uri.setter
    def resource_uri(self, resource_uri):
        """Sets the resource_uri of this AccessRecordResource.

        A resource path which can be top level, fully qualified, or end with a *. Parent resources imply permissions to sub resources.

        :param resource_uri: The resource_uri of this AccessRecordResource.
        :type: str
        """
        if resource_uri is None:
            raise ValueError("Invalid value for `resource_uri`, must not be `None`")

        self._resource_uri = resource_uri



# coding: utf-8

class AccessRecordAccount(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'account_id': 'str'
    }

    attribute_serialization_map = {
        'account_id': 'accountId'
    }

    def __init__(self, account_id=None):
        """AccessRecordAccount"""
        self._account_id = None
        self.account_id = account_id

    @property
    def account_id(self) -> str:
        """Gets the account_id of this AccessRecordAccount.


        :return: The account_id of this AccessRecordAccount.
        :rtype: str
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id: str):
        """Sets the account_id of this AccessRecordAccount.


        :param account_id: The account_id of this AccessRecordAccount.
        :type: str
        """
        self._account_id = account_id



# coding: utf-8

class ResourcePermissionObject(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'action': 'str',
        'allow': 'str'
    }

    attribute_serialization_map = {
        'action': 'action',
        'allow': 'allow'
    }

    def __init__(self, action=None, allow=None):
        """ResourcePermissionObject"""
        self._action = None
        self._allow = None
        self.action = action
        self.allow = allow

    @property
    def action(self):
        """Gets the action of this ResourcePermissionObject.


        :return: The action of this ResourcePermissionObject.
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this ResourcePermissionObject.


        :param action: The action of this ResourcePermissionObject.
        :type: str
        """
        if action is None:
            raise ValueError("Invalid value for `action`, must not be `None`")
        allowed_values = ["CLAIM", "PUBLIC"]
        if action not in allowed_values:
            raise ValueError(
                "Invalid value for `action` ({0}), must be one of {1}"
                .format(action, allowed_values)
            )

        self._action = action

    @property
    def allow(self):
        """Gets the allow of this ResourcePermissionObject.


        :return: The allow of this ResourcePermissionObject.
        :rtype: str
        """
        return self._allow

    @allow.setter
    def allow(self, allow):
        """Sets the allow of this ResourcePermissionObject.


        :param allow: The allow of this ResourcePermissionObject.
        :type: str
        """
        if allow is None:
            raise ValueError("Invalid value for `allow`, must not be `None`")

        self._allow = allow



from __future__ import absolute_import

import unittest

from authress_sdk.api import token_verifier

customDomain = 'authress.token-validation.test'

class TokenVerifierTest(unittest.IsolatedAsyncioTestCase):
  def setUp(self):
    pass

  def tearDown(self):
    pass

  def test_get_token_for_eddsa(self):
    """Test case for verify_token for EdDSA

    Verify that a valid token is generated by the service client token provider and can be verified by the token verifier
    """

    access_key = 'CLIENT.KEY.ACCOUNT.MC4CAQAwBQYDK2VwBCIEIDVjjrIVCH3dVRq4ixRzBwjVHSoB2QzZ2iJuHq1Wshwp'
    publicKey = { 'alg': 'EdDSA', 'kty': 'OKP', 'crv': 'Ed25519', 'x': 'JxtSC5tZZJuaW7Aeu5Kh_3tgCpPZRkHaaFyTj5sQ3KU' }

    identity = token_verifier.TokenVerifier().verify_token(f"https://{customDomain}", access_key, { 'expectedPublicKey': publicKey })
    assert identity['iss'] == f'https://{customDomain}/v1/clients/CLIENT'


# coding: utf-8

class ClaimRequest(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'collection_resource': 'str',
        'resource_id': 'str'
    }

    attribute_serialization_map = {
        'collection_resource': 'collectionResource',
        'resource_id': 'resourceId'
    }

    def __init__(self, collection_resource=None, resource_id=None):
        """ClaimRequest"""
        self._collection_resource = None
        self._resource_id = None
        self.collection_resource = collection_resource
        self.resource_id = resource_id

    @property
    def collection_resource(self):
        """Gets the collection_resource of this ClaimRequest.

        The parent resource to add a child resource to. The resource must have a resource configuration that add the permission CLAIM for this to work.

        :return: The collection_resource of this ClaimRequest.
        :rtype: str
        """
        return self._collection_resource

    @collection_resource.setter
    def collection_resource(self, collection_resource):
        """Sets the collection_resource of this ClaimRequest.

        The parent resource to add a child resource to. The resource must have a resource configuration that add the permission CLAIM for this to work.

        :param collection_resource: The collection_resource of this ClaimRequest.
        :type: str
        """
        if collection_resource is None:
            raise ValueError("Invalid value for `collection_resource`, must not be `None`")

        self._collection_resource = collection_resource

    @property
    def resource_id(self):
        """Gets the resource_id of this ClaimRequest.

        The child resource the user is requesting Admin ownership over.

        :return: The resource_id of this ClaimRequest.
        :rtype: str
        """
        return self._resource_id

    @resource_id.setter
    def resource_id(self, resource_id):
        """Sets the resource_id of this ClaimRequest.

        The child resource the user is requesting Admin ownership over.

        :param resource_id: The resource_id of this ClaimRequest.
        :type: str
        """
        if resource_id is None:
            raise ValueError("Invalid value for `resource_id`, must not be `None`")

        self._resource_id = resource_id



# coding: utf-8

class AccessRecordCollection(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'records': 'list[AccessRecord]'
    }

    attribute_serialization_map = {
        'records': 'records'
    }

    def __init__(self, records=None):
        """AccessRecordCollection"""
        self._records = None
        self.records = records

    @property
    def records(self):
        """Gets the records of this AccessRecordCollection.


        :return: The records of this AccessRecordCollection.
        :rtype: list[AccessRecord]
        """
        return self._records

    @records.setter
    def records(self, records):
        """Sets the records of this AccessRecordCollection.


        :param records: The records of this AccessRecordCollection.
        :type: list[AccessRecord]
        """
        if records is None:
            raise ValueError("Invalid value for `records`, must not be `None`")

        self._records = records



# coding: utf-8

class Account(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'account_id': 'str',
        'created_time': 'datetime',
        'domain': 'str',
        'company': 'object',
        'links': 'object'
    }

    attribute_serialization_map = {
        'account_id': 'accountId',
        'created_time': 'createdTime',
        'domain': 'domain',
        'company': 'company',
        'links': 'links'
    }

    def __init__(self, account_id=None, created_time=None, domain=None, company=None, links=None):
        """Account"""
        self._account_id = None
        self._created_time = None
        self._domain = None
        self._company = None
        self._links = None
        self.account_id = account_id
        self.created_time = created_time
        self.domain = domain
        self.company = company
        self.links = links

    @property
    def account_id(self):
        """Gets the account_id of this Account.


        :return: The account_id of this Account.
        :rtype: str
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        """Sets the account_id of this Account.


        :param account_id: The account_id of this Account.
        :type: str
        """
        self._account_id = account_id

    @property
    def created_time(self):
        """Gets the created_time of this Account.


        :return: The created_time of this Account.
        :rtype: datetime
        """
        return self._created_time

    @created_time.setter
    def created_time(self, created_time):
        """Sets the created_time of this Account.


        :param created_time: The created_time of this Account.
        :type: datetime
        """
        self._created_time = created_time

    @property
    def domain(self):
        """Gets the domain of this Account.

        The top authress sub domain specific for this account to be used with this API.

        :return: The domain of this Account.
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this Account.

        The top authress sub domain specific for this account to be used with this API.

        :param domain: The domain of this Account.
        :type: str
        """
        self._domain = domain

    @property
    def company(self):
        """Gets the company of this Account.


        :return: The company of this Account.
        :rtype: object
        """
        return self._company

    @company.setter
    def company(self, company):
        """Sets the company of this Account.


        :param company: The company of this Account.
        :type: object
        """
        self._company = company

    @property
    def links(self):
        """Gets the links of this Account.


        :return: The links of this Account.
        :rtype: object
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this Account.


        :param links: The links of this Account.
        :type: object
        """
        self._links = links



# coding: utf-8

class UserResources(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'user_id': 'str',
        'resources': 'list[AccessRecordResource]'
    }

    attribute_serialization_map = {
        'user_id': 'userId',
        'resources': 'resources'
    }

    def __init__(self, user_id=None, resources=None):
        """UserResources"""
        self._user_id = None
        self._resources = None
        self.user_id = user_id
        if resources is not None:
            self.resources = resources

    @property
    def user_id(self):
        """Gets the user_id of this UserResources.


        :return: The user_id of this UserResources.
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this UserResources.


        :param user_id: The user_id of this UserResources.
        :type: str
        """
        self._user_id = user_id

    @property
    def resources(self):
        """Gets the resources of this UserResources.

        A list of the resources the user has some permission to.

        :return: The resources of this UserResources.
        :rtype: list[AccessRecordResource]
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this UserResources.

        A list of the resources the user has some permission to.

        :param resources: The resources of this UserResources.
        :type: list[AccessRecordResource]
        """

        self._resources = resources



# coding: utf-8

class Client(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'client_id': 'str',
        'created_time': 'datetime',
        'name': 'str',
        'options': 'ClientOptions'
    }

    attribute_serialization_map = {
        'client_id': 'clientId',
        'created_time': 'createdTime',
        'name': 'name',
        'options': 'options'
    }

    def __init__(self, client_id=None, created_time=None, name=None, options=None):
        """Client"""
        self._client_id = None
        self._created_time = None
        self._name = None
        self._options = None
        self.client_id = client_id
        self.created_time = created_time
        self.name = name
        if options is not None:
            self.options = options

    @property
    def client_id(self):
        """Gets the client_id of this Client.

        The unique id of the client.

        :return: The client_id of this Client.
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """Sets the client_id of this Client.

        The unique id of the client.

        :param client_id: The client_id of this Client.
        :type: str
        """
        self._client_id = client_id

    @property
    def created_time(self):
        """Gets the created_time of this Client.


        :return: The created_time of this Client.
        :rtype: datetime
        """
        return self._created_time

    @created_time.setter
    def created_time(self, created_time):
        """Sets the created_time of this Client.


        :param created_time: The created_time of this Client.
        :type: datetime
        """
        self._created_time = created_time

    @property
    def name(self):
        """Gets the name of this Client.

        The name of the client

        :return: The name of this Client.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Client.

        The name of the client

        :param name: The name of this Client.
        :type: str
        """
        self._name = name

    @property
    def options(self):
        """Gets the options of this Client.


        :return: The options of this Client.
        :rtype: ClientOptions
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this Client.


        :param options: The options of this Client.
        :type: ClientOptions
        """

        self._options = options



# coding: utf-8

class AccessRecord(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'record_id': 'str',
        'name': 'str',
        'account': 'AccessRecordAccount',
        'users': 'list[AccessRecordUser]',
        'groups': 'list[AccessRecordGroup]',
        'admins': 'list[AccessRecordUser]',
        'statements': 'list[AccessRecordStatement]',
        'links': 'object'
    }

    attribute_serialization_map = {
        'record_id': 'recordId',
        'name': 'name',
        'account': 'account',
        'users': 'users',
        'groups': 'groups',
        'admins': 'admins',
        'statements': 'statements',
        'links': 'links'
    }

    def __init__(self, record_id=None, name=None, account=None, users=None, admins=None, statements=None, links=None, groups=None):
        """AccessRecord"""
        self._record_id = None
        self._name = None
        self._account = None
        self._users = None
        self._groups = None
        self._admins = None
        self._statements = None
        self._links = None
        self.record_id = record_id
        self.name = name
        self.account = account
        self.users = users
        self.groups = groups
        self.admins = admins
        self.statements = statements
        self.links = links

    @property
    def record_id(self):
        """Gets the record_id of this AccessRecord.

        Unique identifier for the record, can be specified on record creation.

        :return: The record_id of this AccessRecord.
        :rtype: str
        """
        return self._record_id

    @record_id.setter
    def record_id(self, record_id):
        """Sets the record_id of this AccessRecord.

        Unique identifier for the record, can be specified on record creation.

        :param record_id: The record_id of this AccessRecord.
        :type: str
        """
        self._record_id = record_id

    @property
    def name(self):
        """Gets the name of this AccessRecord.

        A helpful name for this record

        :return: The name of this AccessRecord.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this AccessRecord.

        A helpful name for this record

        :param name: The name of this AccessRecord.
        :type: str
        """
        self._name = name

    @property
    def account(self):
        """Gets the account of this AccessRecord.


        :return: The account of this AccessRecord.
        :rtype: AccessRecordAccount
        """
        return self._account

    @account.setter
    def account(self, account):
        """Sets the account of this AccessRecord.


        :param account: The account of this AccessRecord.
        :type: AccessRecordAccount
        """
        self._account = account

    @property
    def users(self):
        """Gets the users of this AccessRecord.

        The list of users this record applies to

        :return: The users of this AccessRecord.
        :rtype: list[AccessRecordUser]
        """
        return self._users

    @users.setter
    def users(self, users):
        """Sets the users of this AccessRecord.

        The list of users this record applies to

        :param users: The users of this AccessRecord.
        :type: list[AccessRecordUser]
        """
        self._users = users

    @property
    def groups(self):
        """Gets the groups of this AccessRecord.

        The list of groups this record applies to. Users in these groups will be receive access to the resources listed.

        :return: The groups of this AccessRecord.
        :rtype: list[AccessRecordGroup]
        """
        return self._groups

    @groups.setter
    def groups(self, groups):
        """Sets the groups of this AccessRecord.

        The list of groups this record applies to. Users in these groups will be receive access to the resources listed.

        :param groups: The groups of this AccessRecord.
        :type: list[AccessRecordGroup]
        """
        self._groups = groups

    @property
    def admins(self):
        """Gets the admins of this AccessRecord.

        The list of admin that can edit this record even if they do not have global record edit permissions.

        :return: The admins of this AccessRecord.
        :rtype: list[AccessRecordUser]
        """
        return self._admins

    @admins.setter
    def admins(self, admins):
        """Sets the admins of this AccessRecord.

        The list of admin that can edit this record even if they do not have global record edit permissions.

        :param admins: The admins of this AccessRecord.
        :type: list[AccessRecordUser]
        """
        self._admins = admins

    @property
    def statements(self):
        """Gets the statements of this AccessRecord.

        A list of statements which match roles to resources. Users in this record have all statements apply to them

        :return: The statements of this AccessRecord.
        :rtype: list[AccessRecordStatement]
        """
        return self._statements

    @statements.setter
    def statements(self, statements):
        """Sets the statements of this AccessRecord.

        A list of statements which match roles to resources. Users in this record have all statements apply to them

        :param statements: The statements of this AccessRecord.
        :type: list[AccessRecordStatement]
        """
        if statements is None:
            raise ValueError("Invalid value for `statements`, must not be `None`")

        self._statements = statements

    @property
    def links(self):
        """Gets the links of this AccessRecord.


        :return: The links of this AccessRecord.
        :rtype: object
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this AccessRecord.


        :param links: The links of this AccessRecord.
        :type: object
        """
        self._links = links



# coding: utf-8

class ClaimResponse(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
    }

    attribute_serialization_map = {
    }

    def __init__(self):
        """ClaimResponse"""



# coding: utf-8

class UserPermissions(object):

    """
    Attributes:
      openapi_spec_types (dict): The key is attribute name and the value is attribute type.
      attribute_serialization_map (dict): The key is attribute name and the value is json key in definition.
    """
    openapi_spec_types = {
        'user_id': 'str',
        'permissions': 'list[PermissionObject]'
    }

    attribute_serialization_map = {
        'user_id': 'userId',
        'permissions': 'permissions'
    }

    def __init__(self, user_id=None, permissions=None):
        """UserPermissions"""
        self._user_id = None
        self._permissions = None
        self.user_id = user_id
        self.permissions = permissions

    @property
    def user_id(self):
        """Gets the user_id of this UserPermissions.


        :return: The user_id of this UserPermissions.
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this UserPermissions.


        :param user_id: The user_id of this UserPermissions.
        :type: str
        """
        self._user_id = user_id

    @property
    def permissions(self):
        """Gets the permissions of this UserPermissions.

        A list of the permissions

        :return: The permissions of this UserPermissions.
        :rtype: list[PermissionObject]
        """
        return self._permissions

    @permissions.setter
    def permissions(self, permissions):
        """Sets the permissions of this UserPermissions.

        A list of the permissions

        :param permissions: The permissions of this UserPermissions.
        :type: list[PermissionObject]
        """
        if permissions is None:
            raise ValueError("Invalid value for `permissions`, must not be `None`")

        self._permissions = permissions
